﻿using DryIoc;
using Prism.Commands;
using Prism.Mvvm;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows;
using System.Collections.ObjectModel;
using System.Windows.Threading;

namespace Txgy.FilesWatcher.ViewModels
{
    internal class MainViewModel:BindableBase
    {
        public MainViewModel()
        {

            WatcherPath = "E:\\mingzf\\txgy\\WatchChanged";
            InitializeParams();
        }

        private string watcherPath;

        public string WatcherPath
        {
            get { return watcherPath; }
            set { watcherPath = value; }
        }
        private List<string> dataList=new List<string>();

        public List<string> DataList

        {
            get { return dataList; }
            set { dataList = value; }
        }

        private DateTime startTime;
        public DateTime StartTime { get => startTime; set => SetProperty(ref startTime, value); }
        private string runTime;

        public string RunTime
        {
            get { return runTime; }
            set { SetProperty(ref runTime, value); }
        }

        private ObservableCollection<int> intervalTimesSource = new ObservableCollection<int>();

        public ObservableCollection<int> IntervalTimesSource
        {
            get { return intervalTimesSource; }
            set { intervalTimesSource = value; }
        }
        private int selectedIndex;

        public int SelectedIndex
        {
            get { return selectedIndex; }
            set { SetProperty(ref selectedIndex, value); }
        }
        private int proMonInterval = 10;
        public int ProMonInterval
        {
            get => proMonInterval;
            set => SetProperty(ref proMonInterval, value);
        }
        private bool isIndeterminate = false;

        public bool IsIndeterminate
        {
            get { return isIndeterminate; }
            set { SetProperty(ref isIndeterminate, value); }
        }
        public void InitializeParams()
        {
            timer1.Interval = TimeSpan.FromSeconds(ProMonInterval);
            IntervalTimesSource.Add(5);
            IntervalTimesSource.Add(10);
            IntervalTimesSource.Add(15);
            IntervalTimesSource.Add(20);
            IntervalTimesSource.Add(30);
            IntervalTimesSource.Add(60);
            SelectedIndex = 1;

            watcher = new FileSystemWatcher(WatcherPath);
            //初始化监听
            watcher.BeginInit();

            //设置需要监听的更改类型(如:文件或者文件夹的属性,文件或者文件夹的创建时间;NotifyFilters枚举的内容)
            watcher.NotifyFilter = NotifyFilters.Attributes
                                 | NotifyFilters.CreationTime
                                 | NotifyFilters.DirectoryName
                                 | NotifyFilters.FileName
                                 | NotifyFilters.LastAccess
                                 | NotifyFilters.LastWrite
                                 | NotifyFilters.Security
                                 | NotifyFilters.Size;
            //设置监听的路径
            watcher.Path = WatcherPath;
            watcher.Changed += Watcher_Changed;
            watcher.Created += Watcher_Created;
            watcher.Deleted += Watcher_Deleted;
            watcher.Renamed += Watcher_Renamed;
            watcher.Error += OnError;

            //设置监听文件类型
            //   watcher.Filter = "*.txt";
            //设置是否监听子目录
            watcher.IncludeSubdirectories = true;
            //设置是否启用监听
            watcher.EnableRaisingEvents = false;
            watcher.EndInit();

        }

        public DelegateCommand StartCommand => new(Start);

        public DelegateCommand StopCommand => new(Stop);


        private void Start()
        {
            WatchStartOrSopt(true);
        }

        private void Stop()
        {
            WatchStartOrSopt(false);
        }

        public DelegateCommand FilePathSaveCommand => new(() =>
        { 
            System.Windows.Forms.FolderBrowserDialog fbd = new System.Windows.Forms.FolderBrowserDialog();
            fbd.SelectedPath = WatcherPath;
            fbd.ShowNewFolderButton = true;
            if (fbd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
               // MainModel.DataSavePath = fbd.SelectedPath;
                //if (MainModel.DataSavePath != _systemConfig.vpnInfo.DataSavePath)
                //{
                //    updateJson = true;
                //}
            }
        });

        private void OnError(object sender, ErrorEventArgs e)
        {

        }

        private void Watcher_Changed(object sender, FileSystemEventArgs e)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() => {

                DateTime dt = DateTime.Now;
                string tmp = dt.Hour.ToString() + "时" + dt.Minute.ToString() + "分" + dt.Second.ToString() + "秒" + dt.Millisecond.ToString() + "毫秒,目录发生变化\r\n";
                tmp += "改变类型 :" + e.ChangeType.ToString() + "\r\n"; ;
                tmp += "文件全称:" + e.FullPath + "\r\n";
                DataList.Add(tmp);
            }));
        }

        private void Watcher_Renamed(object sender, RenamedEventArgs e)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() => {

                DateTime dt = DateTime.Now;
                string tmp = dt.Hour.ToString() + "时" + dt.Minute.ToString() + "分" + dt.Second.ToString() + "秒" + dt.Millisecond.ToString() + "毫秒,目录发生变化\r\n";
                tmp += "改变类型 :" + e.ChangeType.ToString() + "\r\n"; ;
                tmp += "文件全称:" + e.FullPath + "\r\n";
                DataList.Add(tmp);
            }));
        }

        private void Watcher_Created(object sender, FileSystemEventArgs e)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() => {

                DateTime dt = DateTime.Now;
                string tmp = dt.Hour.ToString() + "时" + dt.Minute.ToString() + "分" + dt.Second.ToString() + "秒" + dt.Millisecond.ToString() + "毫秒,目录发生变化\r\n";
                tmp += "改变类型 :" + e.ChangeType.ToString() + "\r\n"; ;
                tmp += "文件全称:" + e.FullPath + "\r\n";
                DataList.Add(tmp);
            }));
        }

        private void Watcher_Deleted(object sender, FileSystemEventArgs e)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() => {

                DateTime dt = DateTime.Now;
                string tmp = dt.Hour.ToString() + "时" + dt.Minute.ToString() + "分" + dt.Second.ToString() + "秒" + dt.Millisecond.ToString() + "毫秒,目录发生变化\r\n";
                tmp += "改变类型 :" + e.ChangeType.ToString() + "\r\n"; ;
                tmp += "文件全称:" + e.FullPath + "\r\n";
                DataList.Add(tmp);
            }));
        }

        /// <summary>
        /// 启动或者停止监听
        /// </summary>
        /// <param name="IsEnableRaising">True:启用监听,False:关闭监听</param>
        private void WatchStartOrSopt(bool IsEnableRaising)
        {
            watcher.EnableRaisingEvents = IsEnableRaising;
        }
    
        private FileSystemWatcher watcher = new FileSystemWatcher();
        private DispatcherTimer timer1 = new DispatcherTimer();
    }
}
